/**
 * Miranda ICQ contact list and events database to Ickle database converter.
 * $Id: Contact.cpp,v 1.2 2002/05/30 16:00:07 berk Exp $
 *
 * @file  Contact.cpp
 * @brief Contact for particular user.
 *
 * Copyright (c) 2002, Stanislav Karchebny  <berk@upnet.ru>
 * Distributed under BSD License. See file LICENSE for details.
 */

#include <fstream>
#include <string>
#include <unistd.h>
#include "Contact.h"
#include "errwarn.h"
#include "sstream_fix.h"
#include "iconvert.h"

// ickle user info (in brackets size followed by miranda name of setting)
// about = <about information, long text with \n> (s About)
// age = 0 (w Age)
// alias = <alias visible in contact list> (s Nick)
// birth_day = 0 <numeric> (b BirthDay)
// birth_month = 0 <numeric> (b BirthMonth)
// birth_year = 1900 <numeric> (w BirthYear)
// city = (s City)
// country = 0 <numeric> (w Country)
// email = (s e-mail)
// fax = (s Fax)
// firstname = (s FirstName)
// gmt = -100 <numeric> (b Timezone)
// homepage = (s Homepage)
// lang1 = 0 <numeric> (b Language1)
// lang2 = 0 <numeric> (b Language2)
// lang3 = 0 <numeric> (b Language3)
// last_away_msg_check_time = 0
// last_message_time = 0
// last_seen_online_time = 0 (d LastInfoUpdate)
// last_signon_time = 0
// last_status_change_time = 0
// lastname = (s LastName)
// mobile_no = (s Cellular)
// phone = (s Phone)
// sex = 0 (b Gender)
// state = (s State)
// street = (s Street)
// uin = <uin number, decimal> (d UIN)
// zip = (s ZIP)

extern iconvert *converter; // wyrd

/** Add setting to this Contact.
    @param module Name of Miranda module this setting belongs to.
    @param setting Setting data.
*/
void Contact::add(char *module, DBVARIANT *setting)
{
   if( !strncmp(module, "Protocol", strlen("Protocol")) )
   {
      // p == ICQ
      if( !strncmp(setting->name, "p", 1) && setting->type == DBVT_ASCIIZ )
         if( strncmp(setting->pszVal, "ICQ", 3) )
            settings.setValue("uin", 0); // not an ICQ contact, skip it
   }
   else if( !strncmp(module, "ICQ", strlen("ICQ")) )
   {
#define CHECK_NAME(nm) if( !strncmp(setting->name, (nm), strlen((nm))) )
      if( setting->type == DBVT_ASCIIZ )
      {
         CHECK_NAME("About")
         {
            settings.setValue("about", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("Nick")
         {
            settings.setValue("alias", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("City")
         {
            settings.setValue("city", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("e-mail")
         {
            settings.setValue("email", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("Fax")
         {
            settings.setValue("fax", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("FirstName")
         {
            settings.setValue("firstname", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("Homepage")
         {
            settings.setValue("homepage", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("LastName")
         {
            settings.setValue("lastname", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("Cellular")
         {
            settings.setValue("mobile_no", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("Phone")
         {
            settings.setValue("phone", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("State")
         {
            settings.setValue("state", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("Street")
         {
            settings.setValue("street", converter->iconv(string(setting->pszVal)));
         }
         else CHECK_NAME("ZIP")
         {
            settings.setValue("zip", converter->iconv(string(setting->pszVal)));
         }
      }
      else if( setting->type == DBVT_DWORD )
      {
         CHECK_NAME("UIN")
         {
            settings.setValue("uin", setting->dVal);
         }
      }
      else if( setting->type == DBVT_WORD )
      {
         CHECK_NAME("Age")
         {
            settings.setValue("age", setting->wVal);
         }
         else CHECK_NAME("BirthYear")
         {
            settings.setValue("birth_year", setting->wVal);
         }
         else CHECK_NAME("Country")
         {
            settings.setValue("country", setting->wVal);
         }
      }
      else if( setting->type == DBVT_BYTE )
      {
         CHECK_NAME("BirthDay")
         {
            settings.setValue("birth_day", setting->bVal);
         }
         else CHECK_NAME("BirthMonth")
         {
            settings.setValue("birth_month", setting->bVal);
         }
         else CHECK_NAME("Timezone")
         {
            settings.setValue("gmt", setting->bVal);
         }
         else CHECK_NAME("Language1")
         {
            settings.setValue("lang1", setting->bVal);
         }
         else CHECK_NAME("Language2")
         {
            settings.setValue("lang2", setting->bVal);
         }
         else CHECK_NAME("Language3")
         {
            settings.setValue("lang3", setting->bVal);
         }
         else CHECK_NAME("Gender")
         {
            settings.setValue("sex", setting->bVal == 'F' ? 1 : setting->bVal == 'M' ? 2 : 0);
         }
      }
#undef CHECK_NAME
   }
   else if( !strncmp(module, "CList", strlen("CList")) )
   {
      if( !strncmp(setting->name, "LastInfoUpdate", 1) && setting->type == DBVT_DWORD )
         settings.setValue("last_seen_online_time", setting->dVal);
   }
   else warning("unrecognised module %s, skipping setting\n", module);
   // more mods we might want to analyze:
   // Ignore
   // UserOnline
}

/* taken from Ickle History.c */
static string quote_output(const string& text)
{
   ostringstream os;
   int a = 0, l;
   while ( (l = text.find( '\n', a )) != -1 )
   {
      os << text.substr( a, l-a ) << "\\" << endl;
      a = l + 1;
   }
   os << text.substr( a ) << " " << endl;
   return os.str();
}

void Contact::add_history(char *module, DBEvent *event, BYTE *data)
{
   // according to Ickle History.c we log only
   // MESSAGE, URL, SMS, SMS_RECEIPT and EMAIL_EX events
   // history format:
   // "Time: TIMESTAMP\n"
   // "Direction: (Received|Sent)\n"
   // ["Urgent: Yes\n"]
   // for normal message:
   // "Type: Normal\n"
   // "Offline: (Yes|No)\n"
   // "Multiparty: (Yes|No)\n"
   // "Message: QUOTED_MSG\n" (see below)
   // for URLs:
   // "Type: URL\n"
   // "Offline: (Yes|No)\n"
   // "Message: QUOTED_MSG\n"
   // "URL: QUOTED_URL\n"
   // for SMSs:
   // "Type: SMS\n"
   // if (!received) {
   // "Receipt: (Yes|No)\n"
   // }
   // "Message: QUOTED_MSG\n";
   // for SMS Receipts:
   // "Type: SMSReceipt\n"
   // "Delivered: (Yes|No)\n"
   // "Message: QUOTED_MSG\n";
   // for EmailExs:
   // "Type: EmailExpress\n"
   // "Message: QUOTED_MSG\n";

   if( event->eventType == EVENTTYPE_MESSAGE )
   {
      ostringstream msg;
      msg << "Time: " << event->timestamp << endl;
      msg << "Direction: " << (event->flags&DBEF_SENT?"Sent":"Received") << endl;
      msg << "Type: Normal" << endl
          << "Offline: No" << endl     // some guesses here...
          << "Multiparty: No" << endl  // and here...
          << "Message: " << quote_output(converter->iconv((char *)data)) << endl;
      history.push_back(msg.str());
   }
   else if( event->eventType == EVENTTYPE_URL )
   {
      ostringstream msg;
      msg << "Time: " << event->timestamp << endl;
      msg << "Direction: " << (event->flags&DBEF_SENT?"Sent":"Received") << endl;
      msg << "Type: URL" << endl
          << "Offline: No" << endl     // some guesses here...
          << "Message: " << quote_output(converter->iconv((char *)data)) << endl
          << "URL: " << quote_output(converter->iconv((char *)(data + strlen((char *)data) + 1))) << endl;
      history.push_back(msg.str());
   }
   else warning("unsupported event %s, skipped\n", ContactEventType(event));
}

/* write all collected crap into Ickle .user/.history files */
void Contact::write()
{
   DWORD uin = settings.getValueUnsignedInt("uin");
   if( uin != 0 )
   {
      ofstream os;
      string fname;

      // write user file
      do { // dummy loop
      ostringstream userfile;

      if( file_override ) userfile << path << file_override << ".user";
      else                userfile << path << "contacts/" << uin << ".user";
      fname = userfile.str();
      Dnotice("writing user to %s\n", fname.c_str());

      if( !access(fname.c_str(),0) )
      {
         warning("file %s already exists, trying backup solution\n", fname.c_str());
         userfile << "_new";
         fname = userfile.str();
         Dnotice("writing user to %s\n", fname.c_str());
         if( !access(fname.c_str(),0) )
         {
            warning("file %s already exists, skipping\n", fname.c_str());
            break;
         }
      }

      settings.save(fname);
      } while(0); // dummy loop

      // write history file (we do it manually since Ickle's History is not
      // well-suited for writing out such a bunch of lines).
      if(!history.empty())
      {
         ostringstream histfile;

         if( file_override ) histfile << path << file_override << ".history";
         else                histfile << path << "contacts/" << uin << ".history";
         fname = histfile.str();
         Dnotice("writing history to %s\n", fname.c_str());

         if( !access(fname.c_str(),0) )
         {
            warning("file %s already exists, trying backup solution\n", fname.c_str());
            histfile << "_new";
            fname = histfile.str();
            Dnotice("writing history to %s\n", fname.c_str());
            if( !access(fname.c_str(),0) )
            {
               warning("file %s already exists, skipping\n", fname.c_str());
               return;
            }
         }

         os.open(fname.c_str(), ios::out | ios::trunc);
         if(!os.is_open())
         {
            warning("cannot open file %s for writing, skipping\n", fname.c_str());
            return;
         }

         for(vector<string>::iterator i = history.begin(); i != history.end(); i++)
            os << (*i);

         os.close();
      } // if
   }
   else warning("not an ICQ contact, skipped\n");
}
