/**
 * Miranda ICQ contact list and events database to Ickle database converter.
 * $Id: iconvert.cpp,v 1.2 2002/05/30 16:00:07 berk Exp $
 *
 * @file  iconvert.cpp
 * @brief iconv wrapper.
 *
 * Copyright (c) 2002, Stanislav Karchebny  <berk@upnet.ru>
 * Distributed under BSD License. See file LICENSE for details.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <errno.h>
#include "iconvert.h"
#include "errwarn.h"

iconvert::iconvert() : recode_from("")
                     , recode_to("")
#ifdef HAVE_ICONV_OPEN
                     , iconv_state((iconv_t)-1)
#endif
{
}

iconvert::iconvert(const string& from,
                   const string& to) : recode_from(from)
                                     , recode_to(to)
#ifdef HAVE_ICONV_OPEN
                                     , iconv_state((iconv_t)-1)
#endif
{
#ifdef HAVE_ICONV_OPEN
   iconv_state = iconv_open(recode_to.c_str(), recode_from.c_str());
   if( iconv_state == (iconv_t)-1 ) warning("iconvert(): Translation from %s to %s is not supported, will pass-through.\n", recode_from.c_str(), recode_to.c_str());
#endif
}

iconvert::~iconvert()
{
#ifdef HAVE_ICONV_OPEN
   if( iconv_state != (iconv_t)-1 )
      iconv_close(iconv_state);
#endif
}

/* Conversion code is taken from xmms-1.2.7/Input/vorbis/utf8.c
 * Copyright (C) 1999-2001  Haavard Kvaalen <havardk@xmms.org>
 * Distributed under GPL.
 */
string iconvert::iconv(const string& instr)
{
#ifdef HAVE_ICONV_OPEN
   if( iconv_state == (iconv_t)-1 ) return instr;

   const char *str = instr.c_str();

   if (!str)
      return string(NULL);

   size_t outleft, outsize, length;
   char *out, *outptr;
   char *input = (char *)str;

   length = strlen(str);

   /* Due to a GLIBC bug, round outbuf_size up to a multiple of 4 */
   /* + 1 for nul in case len == 1 */
   outsize = ((length + 3) & ~3) + 1;
   out = (char *)malloc(outsize);
   outleft = outsize - 1;
   outptr = out;

   // reset iconv to initial state
   ::iconv( iconv_state, NULL, NULL, NULL, NULL );

 retry:
   if( ::iconv(iconv_state, &input, &length, &outptr, &outleft) == (size_t)-1 )
   {
      int used;
      switch (errno)
      {
         case E2BIG:
            used = outptr - out;
            outsize = (outsize - 1) * 2 + 1;
            out = (char *)realloc(out, outsize);
            outptr = out + used;
            outleft = outsize - 1 - used;
            goto retry;
         case EINVAL:
            break;
         case EILSEQ:
            /* Invalid sequence, try to get the rest of the string */
            input++;
            length = strlen(input);
            goto retry;
         default:
            warning("iconvert.iconv(): Conversion failed. Input string: %s; Error: %s\n", str, strerror(errno));
            break;
      }
   }
   *outptr = '\0';

   return string(out);
#else
   return instr;
#endif
}
