/**
 * Miranda ICQ contact list and events database to Ickle database converter.
 * $Id: options.h,v 1.6 2002/05/30 16:00:07 berk Exp $
 *
 * @file  options.h
 * @brief Generic Options Support Header File
 *
 * Copyright (c) 2001, Stanislav Karchebny  <berk@upnet.ru>
 * Distributed under BSD License. See file LICENSE for details.
 */

#ifndef __M2I_OPTIONS_H
#define __M2I_OPTIONS_H

/* an option structure
 * operate on either -sopt, --lopt, -sopt <val> or --lopt=<val>
 */
typedef struct opt_option_s
{
    /* short option letter if present, 0 otherwise */
    char sopt;

    /* long option name if present, NULL otherwise */
    /*@null@*/ const char *lopt;

    /* !=0 if option requires parameter, 0 if not */
    int takes_param;

    int (*handler) (char *cmd, /*@null@*/ char *param, int extra);
    int extra;         /* extra value for handler */
    const char *description;   /* description to use in help_msg() */

    /* optional description for the param taken (NULL if not present) */
    /*  (short - will be printed after option sopt/lopt) */
    /*@null@*/ const char *param_desc;
} opt_option;

#ifdef __cplusplus
extern "C" {
#endif

/* handle everything that is not an option */
int not_an_option_handler(char *param);

/* parse command line calling handlers when appropriate
 * argc, argv - pass directly from main(argc,argv)
 * options - array of options
 * nopts - options count
 */
int parse_cmdline(int argc, char **argv, opt_option *options, int nopts);

/* display help message msg followed by list of options in options and followed
 * by tail
 */
void help_msg(const char *msg, const char *tail, opt_option *options,
         int nopts);

#ifdef __cplusplus
}
#endif

#endif
